{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "2e1f5f65-9726-4df6-9fc5-21b3e375e1f2",
   "metadata": {},
   "source": [
    "# Using Memory Tiles in your Application\n",
    "\n",
    "## Goals\n",
    "\n",
    "* Learn how to specify and use the memory tile in the application callgraph\n",
    "\n",
    "* Learn how to specify multiple kernels in an application\n",
    "\n",
    "* Build a scaled-up version of the design using the split and concatenate operations of the memory tile \n",
    "\n",
    "---"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "2353a0b5-47e0-4ab7-98fd-7ed19d882539",
   "metadata": {},
   "source": [
    "## Introduction\n",
    "\n",
    "In this example you will learn more about the functionality of the memory tile and explore how it can be used. \n",
    "\n",
    "In the previous example we used one compute tile to run a pre-build RGBA threshold. In this example we will scale up and use four compute tiles to run four different instances of the same RGBA threshold function.\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "a3d9edb2-0fb3-4013-bb66-e488ebb1432f",
   "metadata": {},
   "source": [
    "## Create the Example Application \n",
    "\n",
    "Create the application using the `RgbaRtpThres()` software kernel. Notice that as before the example will process 720p RGBA images (1280x720) and we will pass one row of the image at a time to the software kernel. \n",
    "\n",
    "The threshold function C++ signature is:\n",
    "\n",
    "```c++\n",
    "   RgbaRtpThres([uint8_t* input_buffer], [uint16_t input_in_bytes], [uint8_t red_threshold], [uint8_t blue_threshold], [uint8_t green_threshold])\n",
    "```\n",
    "\n",
    "The initial RGB threshold values passed to the threshold function in the cell below are [128,128,128]."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "id": "cffae655-e854-4095-b60a-a488fc17feaf",
   "metadata": {
    "tags": []
   },
   "outputs": [],
   "source": [
    "from npu.lib import RgbaRtpThres\n",
    "from npu.build.appbuilder import AppBuilder\n",
    "import numpy as np\n",
    "\n",
    "class SimpleApplication(AppBuilder):\n",
    "\n",
    "    def __init__(self):\n",
    "        self.k = RgbaRtpThres()\n",
    "        super().__init__()\n",
    "\n",
    "    def callgraph(self, x_in:np.ndarray, x_out:np.ndarray) -> None:\n",
    "        rows = x_in.shape[0]\n",
    "        bytes_per_row = x_in.shape[1] * x_in.shape[2]\n",
    "        for row in range(rows):\n",
    "            x = self.k(x_in[row], bytes_per_row, 128, 128, 128)\n",
    "            x_out[row] = x"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "e8729df5-a38b-44f9-965f-08ad76dda882",
   "metadata": {},
   "source": [
    "### Visualize the application\n",
    "\n",
    "Allocate memory buffers, and visualize the application using the Riallto `display()` method."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "67710083-2d0b-4c76-b25f-7e5e1357ef98",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "image/svg+xml": [
       "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" id=\"svg\" version=\"1.1\" width=\"420\" height=\"980\" viewBox=\"0, 0, 420, 980\">\n",
       "            <defs>\n",
       "                <marker id=\"ic_startarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"10\" refY=\"3.5\" orient=\"auto\">\n",
       "                <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "                <marker id=\"ic_endarrow\" markerWidth=\"7\" markerHeight=\"4\" refX=\"0\" refY=\"2\" orient=\"auto\" markerUnits=\"strokeWidth\">\n",
       "                    <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "\n",
       "                <marker id=\"red_startarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"0\" refY=\"2\" orient=\"auto\">\n",
       "                <polygon points=\"0 2, 4 0, 4 4\" fill=\"red\"/>\n",
       "                </marker>\n",
       "                <marker id=\"red_endarrow\" markerWidth=\"7\" markerHeight=\"4\" refX=\"4\" refY=\"2\" orient=\"auto\" markerUnits=\"strokeWidth\">\n",
       "                    <polygon points=\"0 0, 4 2, 0 4\" fill=\"red\"/>\n",
       "                </marker>\n",
       "\n",
       "                <marker id=\"startarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"10\" refY=\"3.5\" orient=\"auto\">\n",
       "                <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "                <marker id=\"endarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"0\" refY=\"3.5\" orient=\"auto\" markerUnits=\"strokeWidth\">\n",
       "                    <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "            </defs>\n",
       "            <style>                .ic_animations {                     opacity: 0.8;                 }            </style>\n",
       "            <g id=\"svg\"><rect id=\"border\" class=\"border\" width=\"400\" height=\"960\" x=\"10.0\" y=\"10.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/>\n",
       "<g> <rect id=\"aie_tile0\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"20\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile0\" class=\"aie_tile_text\" x=\"20\" y=\"20\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"89.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box0\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"30\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box0\" class=\"interconnect_box_text\" x=\"30\" y=\"30\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"49.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line0_0\" class=\"interconnect\" x1=\"38.0\" y1=\"45.0\" x2=\"58.0\" y2=\"45.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line0_1\" class=\"interconnect\" x1=\"62.0\" y1=\"55.0\" x2=\"42.0\" y2=\"55.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line0_2\" class=\"interconnect\" x1=\"45.0\" y1=\"62.0\" x2=\"45.0\" y2=\"42.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line0_3\" class=\"interconnect\" x1=\"55.0\" y1=\"38.0\" x2=\"55.0\" y2=\"58.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container0\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"90.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container0\" class=\"aie_tile_container_text\" x=\"70\" y=\"90.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"119.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box0\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"97.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box0\" class=\"aie_box_text\" x=\"77.0\" y=\"97.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"94.5\" y=\"105.6\" dy=\"1em\" font-size=\"12\">AI </tspan><tspan x=\"80.5\" y=\"117.6\" dy=\"1em\" font-size=\"12\">Engine </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box0\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"97.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box0\" class=\"local_memory_box_text\" x=\"137.0\" y=\"97.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"147.0\" y=\"105.6\" dy=\"1em\" font-size=\"12\">Data </tspan><tspan x=\"138.0\" y=\"117.6\" dy=\"1em\" font-size=\"12\">Memory </tspan></text></g>\n",
       "<g> <rect id=\"aie_tile1\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"180\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile1\" class=\"aie_tile_text\" x=\"20\" y=\"180\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"249.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box1\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"190\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box1\" class=\"interconnect_box_text\" x=\"30\" y=\"190\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"209.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line1_0\" class=\"interconnect\" x1=\"38.0\" y1=\"205.0\" x2=\"58.0\" y2=\"205.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line1_1\" class=\"interconnect\" x1=\"62.0\" y1=\"215.0\" x2=\"42.0\" y2=\"215.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line1_2\" class=\"interconnect\" x1=\"45.0\" y1=\"222.0\" x2=\"45.0\" y2=\"202.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line1_3\" class=\"interconnect\" x1=\"55.0\" y1=\"198.0\" x2=\"55.0\" y2=\"218.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container1\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"250.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container1\" class=\"aie_tile_container_text\" x=\"70\" y=\"250.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"279.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box1\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"257.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box1\" class=\"aie_box_text\" x=\"77.0\" y=\"257.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"94.5\" y=\"265.6\" dy=\"1em\" font-size=\"12\">AI </tspan><tspan x=\"80.5\" y=\"277.6\" dy=\"1em\" font-size=\"12\">Engine </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box1\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"257.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box1\" class=\"local_memory_box_text\" x=\"137.0\" y=\"257.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"147.0\" y=\"265.6\" dy=\"1em\" font-size=\"12\">Data </tspan><tspan x=\"138.0\" y=\"277.6\" dy=\"1em\" font-size=\"12\">Memory </tspan></text></g>\n",
       "<g> <rect id=\"aie_tile2\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"340\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile2\" class=\"aie_tile_text\" x=\"20\" y=\"340\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"409.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box2\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"350\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box2\" class=\"interconnect_box_text\" x=\"30\" y=\"350\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"369.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line2_0\" class=\"interconnect\" x1=\"38.0\" y1=\"365.0\" x2=\"58.0\" y2=\"365.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line2_1\" class=\"interconnect\" x1=\"62.0\" y1=\"375.0\" x2=\"42.0\" y2=\"375.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line2_2\" class=\"interconnect\" x1=\"45.0\" y1=\"382.0\" x2=\"45.0\" y2=\"362.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line2_3\" class=\"interconnect\" x1=\"55.0\" y1=\"358.0\" x2=\"55.0\" y2=\"378.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container2\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"410.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container2\" class=\"aie_tile_container_text\" x=\"70\" y=\"410.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"439.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box2\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"417.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box2\" class=\"aie_box_text\" x=\"77.0\" y=\"417.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"94.5\" y=\"425.6\" dy=\"1em\" font-size=\"12\">AI </tspan><tspan x=\"80.5\" y=\"437.6\" dy=\"1em\" font-size=\"12\">Engine </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box2\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"417.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box2\" class=\"local_memory_box_text\" x=\"137.0\" y=\"417.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"147.0\" y=\"425.6\" dy=\"1em\" font-size=\"12\">Data </tspan><tspan x=\"138.0\" y=\"437.6\" dy=\"1em\" font-size=\"12\">Memory </tspan></text></g>\n",
       "<g> <rect id=\"aie_tile3\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"500\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile3\" class=\"aie_tile_text\" x=\"20\" y=\"500\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"569.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box3\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"510\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box3\" class=\"interconnect_box_text\" x=\"30\" y=\"510\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"529.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line3_0\" class=\"interconnect\" x1=\"38.0\" y1=\"525.0\" x2=\"58.0\" y2=\"525.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line3_1\" class=\"interconnect\" x1=\"62.0\" y1=\"535.0\" x2=\"42.0\" y2=\"535.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line3_2\" class=\"interconnect\" x1=\"45.0\" y1=\"542.0\" x2=\"45.0\" y2=\"522.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line3_3\" class=\"interconnect\" x1=\"55.0\" y1=\"518.0\" x2=\"55.0\" y2=\"538.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container3\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"570.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container3\" class=\"aie_tile_container_text\" x=\"70\" y=\"570.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"599.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box3\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"577.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box3\" class=\"aie_box_text\" x=\"77.0\" y=\"577.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"100.0\" y=\"591.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box3\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"577.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box3\" class=\"local_memory_box_text\" x=\"137.0\" y=\"577.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"160.0\" y=\"591.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"mem_tile4\" class=\"mem_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"660\" stroke=\"black\" stroke-width=\"2\" fill=\"#E0FFC2\"/> </g>\n",
       "<g> <text id=\"text_mem_tile4\" class=\"mem_tile_text\" x=\"20\" y=\"660\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"729.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box4\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"670\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box4\" class=\"interconnect_box_text\" x=\"30\" y=\"670\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"689.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line4_0\" class=\"interconnect\" x1=\"38.0\" y1=\"685.0\" x2=\"58.0\" y2=\"685.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line4_1\" class=\"interconnect\" x1=\"62.0\" y1=\"695.0\" x2=\"42.0\" y2=\"695.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line4_2\" class=\"interconnect\" x1=\"45.0\" y1=\"702.0\" x2=\"45.0\" y2=\"682.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line4_3\" class=\"interconnect\" x1=\"55.0\" y1=\"678.0\" x2=\"55.0\" y2=\"698.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"mem_tile_container4\" class=\"mem_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"730.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_mem_tile_container4\" class=\"mem_tile_container_text\" x=\"70\" y=\"730.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"759.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_16\" x=\"77.0\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_16\" x=\"83.625\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_16\" x=\"90.25\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_16\" x=\"96.875\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_16\" x=\"103.5\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_16\" x=\"110.125\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_16\" x=\"116.75\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_16\" x=\"123.375\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_8_16\" x=\"130.0\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_9_16\" x=\"136.625\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_10_16\" x=\"143.25\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_11_16\" x=\"149.875\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_12_16\" x=\"156.5\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_13_16\" x=\"163.125\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_14_16\" x=\"169.75\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_15_16\" x=\"176.375\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"mem_tile_memory4\" class=\"mem_tile_memory\" width=\"106\" height=\"46.0\" x=\"77\" y=\"737.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_mem_tile_memory4\" class=\"mem_tile_memory_text\" x=\"77\" y=\"737.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"94.0\" y=\"744.4\" dy=\"1em\" font-size=\"13\">Memory Tile </tspan><tspan x=\"93.0\" y=\"757.4\" dy=\"1em\" font-size=\"13\">Data movers </tspan></text></g>\n",
       "<g> <rect id=\"if_tile5\" class=\"if_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"820\" stroke=\"black\" stroke-width=\"2\" fill=\"#E0C2FF\"/> </g>\n",
       "<g> <text id=\"text_if_tile5\" class=\"if_tile_text\" x=\"20\" y=\"820\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"889.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box5\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"830\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box5\" class=\"interconnect_box_text\" x=\"30\" y=\"830\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"849.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line5_0\" class=\"interconnect\" x1=\"38.0\" y1=\"845.0\" x2=\"58.0\" y2=\"845.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line5_1\" class=\"interconnect\" x1=\"62.0\" y1=\"855.0\" x2=\"42.0\" y2=\"855.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line5_2\" class=\"interconnect\" x1=\"45.0\" y1=\"862.0\" x2=\"45.0\" y2=\"842.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line5_3\" class=\"interconnect\" x1=\"55.0\" y1=\"838.0\" x2=\"55.0\" y2=\"858.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"if_tile_container5\" class=\"if_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"890.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_if_tile_container5\" class=\"if_tile_container_text\" x=\"70\" y=\"890.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"919.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"if_dmabox5\" class=\"if_dmabox\" width=\"106\" height=\"46.0\" x=\"77\" y=\"897.0\" stroke=\"black\" stroke-width=\"\" fill=\"white\"/><animate attributeName=\"stroke-width\" dur=\"0.5s\" repeatCount=\"indefinite\" values=\"0;                       2;0\"/>\n",
       " </g>\n",
       "<g> <text id=\"text_if_dmabox5\" class=\"if_dmabox_text\" x=\"77\" y=\"897.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"92.0\" y=\"904.4\" dy=\"1em\" font-size=\"13\">Interface Tile </tspan><tspan x=\"93.0\" y=\"917.4\" dy=\"1em\" font-size=\"13\">Data movers </tspan></text></g>\n",
       "<line id=\"diagonal5\" class=\"interconnect\" x1=\"70\" y1=\"870\" x2=\"90.0\" y2=\"890.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<path id=\"south_mem_connection5\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" d=\"M 130.0, 950.0,                 l0, 30\"/>\n",
       "<circle id=\"d_circle5\" class=\"ic_animations\" cx=\"90.0\" cy=\"890.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle5\" class=\"ic_animations\" cx=\"70\" cy=\"870\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"mem_circle_east5\" class=\"mem_animations\" cx=\"0\" cy=\"0\" r=\"3\" fill=\"none\">                    <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" path=\"M 130.0, 980,                         l 0, -30                        \" begin=\"0s\"/>                    <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>                    </circle>\n",
       "<circle id=\"mem_circle_east5\" class=\"mem_animations\" cx=\"0\" cy=\"0\" r=\"3\" fill=\"none\">                    <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" path=\"M 130.0, 950,                         l 0, 30                        \" begin=\"0s\"/>                    <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>                    </circle>\n",
       "<line id=\"diagonal0\" class=\"interconnect\" x1=\"70\" y1=\"70\" x2=\"90.0\" y2=\"90.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection0\" class=\"interconnect\" x1=\"50\" y1=\"70\" x2=\"50\" y2=\"190\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal1\" class=\"interconnect\" x1=\"70\" y1=\"230\" x2=\"90.0\" y2=\"250.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection1\" class=\"interconnect\" x1=\"50\" y1=\"230\" x2=\"50\" y2=\"350\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal2\" class=\"interconnect\" x1=\"70\" y1=\"390\" x2=\"90.0\" y2=\"410.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection2\" class=\"interconnect\" x1=\"50\" y1=\"390\" x2=\"50\" y2=\"510\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal3\" class=\"interconnect\" x1=\"70\" y1=\"550\" x2=\"90.0\" y2=\"570.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection3\" class=\"interconnect\" x1=\"50\" y1=\"550\" x2=\"50\" y2=\"670\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal4\" class=\"interconnect\" x1=\"70\" y1=\"710\" x2=\"90.0\" y2=\"730.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection4\" class=\"interconnect\" x1=\"50\" y1=\"710\" x2=\"50\" y2=\"830\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<circle id=\"d_circle3\" class=\"ic_animations\" cx=\"70\" cy=\"550\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north3\" class=\"ic_animations\" cx=\"50\" cy=\"670\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle3\" class=\"ic_animations\" cx=\"90.0\" cy=\"570.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south3\" class=\"ic_animations\" cx=\"50\" cy=\"550\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north4\" class=\"ic_animations\" cx=\"50\" cy=\"830\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south4\" class=\"ic_animations\" cx=\"50\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<path id=\"south_mem_connection0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 160.0, 143.0,                 l0, 80.0,                l-30.0, 17.0,                l-30.0, 17.0\"/>\n",
       "<path id=\"south_mem_connection0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 100.0, 143.0,                 l0, 80.0,                l30.0, 17.0,                l30.0, 17.0\"/>\n",
       "<line id=\"internal_mem_connection0\" x1=\"123.0\" y1=\"120.0\" x2=\"137.0\" y2=\"120.0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<path id=\"south_mem_connection1\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 160.0, 303.0,                 l0, 80.0,                l-30.0, 17.0,                l-30.0, 17.0\"/>\n",
       "<path id=\"south_mem_connection1\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 100.0, 303.0,                 l0, 80.0,                l30.0, 17.0,                l30.0, 17.0\"/>\n",
       "<line id=\"internal_mem_connection1\" x1=\"123.0\" y1=\"280.0\" x2=\"137.0\" y2=\"280.0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<path id=\"south_mem_connection2\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 160.0, 463.0,                 l0, 80.0,                l-30.0, 17.0,                l-30.0, 17.0\"/>\n",
       "<path id=\"south_mem_connection2\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 100.0, 463.0,                 l0, 80.0,                l30.0, 17.0,                l30.0, 17.0\"/>\n",
       "<line id=\"internal_mem_connection2\" x1=\"123.0\" y1=\"440.0\" x2=\"137.0\" y2=\"440.0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<linearGradient id=\"kernel_3\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#C00000\"/> </linearGradient> <circle id=\"kernel_circle_3\" class=\"kernel\" cx=\"100.0\" cy=\"600.0\" r=\"20\" fill=\"url(#kernel_3)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 100.0 600.0\" to=\"360 100.0 600.0\" dur=\"4s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<g transform=\"rotate(180, 139.875, 623.0) \"> <rect id=\"buffer3_0\" class=\"aie_tile_buffers\" x=\"137.0\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C36D6D;#C36D6D;#C36D6D;#C36D6D\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 145.625, 623.0) \"> <rect id=\"buffer3_1\" class=\"aie_tile_buffers\" x=\"142.75\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C36D6D;#C36D6D;#C36D6D;#C36D6D\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 151.375, 623.0) \"> <rect id=\"buffer3_2\" class=\"aie_tile_buffers\" x=\"148.5\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C00000;#C00000;#C00000;#C00000\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 157.125, 623.0) \"> <rect id=\"buffer3_3\" class=\"aie_tile_buffers\" x=\"154.25\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C00000;#C00000;#C00000;#C00000\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<linearGradient id=\"kernel_1001\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#C00000\"/> </linearGradient> <circle id=\"kernel_circle_1001\" class=\"kernel\" cx=\"260.0\" cy=\"940.0\" r=\"20\" fill=\"url(#kernel_1001)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 260.0 940.0\" to=\"360 260.0 940.0\" dur=\"0s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<text class=\"text\" x=\"288\" y=\"945.0\">rgba_rtp_thresh_0</text></g></svg>"
      ],
      "text/plain": [
       "<IPython.core.display.SVG object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "app_builder = SimpleApplication()\n",
    "\n",
    "x_in = np.zeros(shape=(720, 1280, 4), dtype=np.uint8)\n",
    "x_out = np.zeros(shape=(720, 1280, 4), dtype=np.uint8)\n",
    "app_builder(x_in, x_out)\n",
    "\n",
    "app_builder.display()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "8cb8607c-3aaf-46d7-a43f-d78d91442e2d",
   "metadata": {},
   "source": [
    "Notice the placement of the threshold software kernel and that data moves from the interface tile directly to the compute tile without using the memory tile data buffers."
   ]
  },
  {
   "cell_type": "markdown",
   "id": "f7c625cc-a41c-419e-ac15-df41bfa8b05b",
   "metadata": {},
   "source": [
    "## Add the Memory Tile into Previous Application"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "80134eb5-c12f-464d-a2cf-69dc9a7adf56",
   "metadata": {},
   "source": [
    "The following application code will be used to explore some of the memory tile operations. "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "id": "97ff1557-b6a1-41d1-bb03-baf2a2d1c6dc",
   "metadata": {
    "tags": []
   },
   "outputs": [],
   "source": [
    "from npu.build.mtkernel import MTPassThrough\n",
    "\n",
    "class SimpleMemTileApplication(AppBuilder):\n",
    "\n",
    "    def __init__(self):\n",
    "        self.k = RgbaRtpThres()\n",
    "        self.mtbuffer_in = MTPassThrough()\n",
    "        self.mtbuffer_out = MTPassThrough()\n",
    "        super().__init__()\n",
    "\n",
    "    def callgraph(self, x_in:np.ndarray, x_out:np.ndarray) -> None:\n",
    "        rows = x_in.shape[0]\n",
    "        bytes_per_row = x_in.shape[1] * x_in.shape[2]\n",
    "        for row in range(rows):\n",
    "            mtpin = self.mtbuffer_in(x_in[row])\n",
    "            x = self.k(mtpin, bytes_per_row, 128, 128, 128)\n",
    "            mtpout = self.mtbuffer_out(x)\n",
    "            x_out[row] = mtpout"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "dc9fd34b",
   "metadata": {},
   "source": [
    "Rather than start by examining the code in detail, let us look at the visualization of this application first and we will return to the code after that.  "
   ]
  },
  {
   "cell_type": "markdown",
   "id": "8a716366-d7f5-4d60-a1a9-f87b52f7d006",
   "metadata": {},
   "source": [
    "### Visualize with the memory tile application"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "id": "ca357b2b-5a3c-43b7-b42d-94fcb8035587",
   "metadata": {
    "tags": []
   },
   "outputs": [],
   "source": [
    "app_builder = SimpleMemTileApplication()\n",
    "app_builder(x_in, x_out)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "id": "fb5b3cac-87ac-4a71-89e0-96c16997b90b",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "image/svg+xml": [
       "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" id=\"svg\" version=\"1.1\" width=\"420\" height=\"980\" viewBox=\"0, 0, 420, 980\">\n",
       "            <defs>\n",
       "                <marker id=\"ic_startarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"10\" refY=\"3.5\" orient=\"auto\">\n",
       "                <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "                <marker id=\"ic_endarrow\" markerWidth=\"7\" markerHeight=\"4\" refX=\"0\" refY=\"2\" orient=\"auto\" markerUnits=\"strokeWidth\">\n",
       "                    <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "\n",
       "                <marker id=\"red_startarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"0\" refY=\"2\" orient=\"auto\">\n",
       "                <polygon points=\"0 2, 4 0, 4 4\" fill=\"red\"/>\n",
       "                </marker>\n",
       "                <marker id=\"red_endarrow\" markerWidth=\"7\" markerHeight=\"4\" refX=\"4\" refY=\"2\" orient=\"auto\" markerUnits=\"strokeWidth\">\n",
       "                    <polygon points=\"0 0, 4 2, 0 4\" fill=\"red\"/>\n",
       "                </marker>\n",
       "\n",
       "                <marker id=\"startarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"10\" refY=\"3.5\" orient=\"auto\">\n",
       "                <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "                <marker id=\"endarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"0\" refY=\"3.5\" orient=\"auto\" markerUnits=\"strokeWidth\">\n",
       "                    <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "            </defs>\n",
       "            <style>                .ic_animations {                     opacity: 0.8;                 }            </style>\n",
       "            <g id=\"svg\"><rect id=\"border\" class=\"border\" width=\"400\" height=\"960\" x=\"10.0\" y=\"10.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/>\n",
       "<g> <rect id=\"aie_tile0\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"20\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile0\" class=\"aie_tile_text\" x=\"20\" y=\"20\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"89.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box0\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"30\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box0\" class=\"interconnect_box_text\" x=\"30\" y=\"30\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"49.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line0_0\" class=\"interconnect\" x1=\"38.0\" y1=\"45.0\" x2=\"58.0\" y2=\"45.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line0_1\" class=\"interconnect\" x1=\"62.0\" y1=\"55.0\" x2=\"42.0\" y2=\"55.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line0_2\" class=\"interconnect\" x1=\"45.0\" y1=\"62.0\" x2=\"45.0\" y2=\"42.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line0_3\" class=\"interconnect\" x1=\"55.0\" y1=\"38.0\" x2=\"55.0\" y2=\"58.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container0\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"90.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container0\" class=\"aie_tile_container_text\" x=\"70\" y=\"90.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"119.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box0\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"97.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box0\" class=\"aie_box_text\" x=\"77.0\" y=\"97.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"94.5\" y=\"105.6\" dy=\"1em\" font-size=\"12\">AI </tspan><tspan x=\"80.5\" y=\"117.6\" dy=\"1em\" font-size=\"12\">Engine </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box0\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"97.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box0\" class=\"local_memory_box_text\" x=\"137.0\" y=\"97.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"147.0\" y=\"105.6\" dy=\"1em\" font-size=\"12\">Data </tspan><tspan x=\"138.0\" y=\"117.6\" dy=\"1em\" font-size=\"12\">Memory </tspan></text></g>\n",
       "<g> <rect id=\"aie_tile1\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"180\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile1\" class=\"aie_tile_text\" x=\"20\" y=\"180\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"249.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box1\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"190\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box1\" class=\"interconnect_box_text\" x=\"30\" y=\"190\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"209.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line1_0\" class=\"interconnect\" x1=\"38.0\" y1=\"205.0\" x2=\"58.0\" y2=\"205.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line1_1\" class=\"interconnect\" x1=\"62.0\" y1=\"215.0\" x2=\"42.0\" y2=\"215.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line1_2\" class=\"interconnect\" x1=\"45.0\" y1=\"222.0\" x2=\"45.0\" y2=\"202.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line1_3\" class=\"interconnect\" x1=\"55.0\" y1=\"198.0\" x2=\"55.0\" y2=\"218.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container1\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"250.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container1\" class=\"aie_tile_container_text\" x=\"70\" y=\"250.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"279.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box1\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"257.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box1\" class=\"aie_box_text\" x=\"77.0\" y=\"257.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"94.5\" y=\"265.6\" dy=\"1em\" font-size=\"12\">AI </tspan><tspan x=\"80.5\" y=\"277.6\" dy=\"1em\" font-size=\"12\">Engine </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box1\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"257.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box1\" class=\"local_memory_box_text\" x=\"137.0\" y=\"257.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"147.0\" y=\"265.6\" dy=\"1em\" font-size=\"12\">Data </tspan><tspan x=\"138.0\" y=\"277.6\" dy=\"1em\" font-size=\"12\">Memory </tspan></text></g>\n",
       "<g> <rect id=\"aie_tile2\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"340\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile2\" class=\"aie_tile_text\" x=\"20\" y=\"340\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"409.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box2\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"350\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box2\" class=\"interconnect_box_text\" x=\"30\" y=\"350\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"369.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line2_0\" class=\"interconnect\" x1=\"38.0\" y1=\"365.0\" x2=\"58.0\" y2=\"365.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line2_1\" class=\"interconnect\" x1=\"62.0\" y1=\"375.0\" x2=\"42.0\" y2=\"375.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line2_2\" class=\"interconnect\" x1=\"45.0\" y1=\"382.0\" x2=\"45.0\" y2=\"362.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line2_3\" class=\"interconnect\" x1=\"55.0\" y1=\"358.0\" x2=\"55.0\" y2=\"378.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container2\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"410.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container2\" class=\"aie_tile_container_text\" x=\"70\" y=\"410.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"439.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box2\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"417.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box2\" class=\"aie_box_text\" x=\"77.0\" y=\"417.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"94.5\" y=\"425.6\" dy=\"1em\" font-size=\"12\">AI </tspan><tspan x=\"80.5\" y=\"437.6\" dy=\"1em\" font-size=\"12\">Engine </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box2\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"417.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box2\" class=\"local_memory_box_text\" x=\"137.0\" y=\"417.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"147.0\" y=\"425.6\" dy=\"1em\" font-size=\"12\">Data </tspan><tspan x=\"138.0\" y=\"437.6\" dy=\"1em\" font-size=\"12\">Memory </tspan></text></g>\n",
       "<g> <rect id=\"aie_tile3\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"500\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile3\" class=\"aie_tile_text\" x=\"20\" y=\"500\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"569.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box3\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"510\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box3\" class=\"interconnect_box_text\" x=\"30\" y=\"510\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"529.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line3_0\" class=\"interconnect\" x1=\"38.0\" y1=\"525.0\" x2=\"58.0\" y2=\"525.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line3_1\" class=\"interconnect\" x1=\"62.0\" y1=\"535.0\" x2=\"42.0\" y2=\"535.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line3_2\" class=\"interconnect\" x1=\"45.0\" y1=\"542.0\" x2=\"45.0\" y2=\"522.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line3_3\" class=\"interconnect\" x1=\"55.0\" y1=\"518.0\" x2=\"55.0\" y2=\"538.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container3\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"570.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container3\" class=\"aie_tile_container_text\" x=\"70\" y=\"570.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"599.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box3\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"577.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box3\" class=\"aie_box_text\" x=\"77.0\" y=\"577.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"100.0\" y=\"591.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box3\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"577.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box3\" class=\"local_memory_box_text\" x=\"137.0\" y=\"577.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"160.0\" y=\"591.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"mem_tile4\" class=\"mem_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"660\" stroke=\"black\" stroke-width=\"2\" fill=\"#E0FFC2\"/> </g>\n",
       "<g> <text id=\"text_mem_tile4\" class=\"mem_tile_text\" x=\"20\" y=\"660\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"729.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box4\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"670\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box4\" class=\"interconnect_box_text\" x=\"30\" y=\"670\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"689.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line4_0\" class=\"interconnect\" x1=\"38.0\" y1=\"685.0\" x2=\"58.0\" y2=\"685.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line4_1\" class=\"interconnect\" x1=\"62.0\" y1=\"695.0\" x2=\"42.0\" y2=\"695.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line4_2\" class=\"interconnect\" x1=\"45.0\" y1=\"702.0\" x2=\"45.0\" y2=\"682.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line4_3\" class=\"interconnect\" x1=\"55.0\" y1=\"678.0\" x2=\"55.0\" y2=\"698.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"mem_tile_container4\" class=\"mem_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"730.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_mem_tile_container4\" class=\"mem_tile_container_text\" x=\"70\" y=\"730.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"759.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_16\" x=\"77.0\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_16\" x=\"83.625\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_16\" x=\"90.25\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_16\" x=\"96.875\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_16\" x=\"103.5\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_16\" x=\"110.125\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_16\" x=\"116.75\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_16\" x=\"123.375\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_8_16\" x=\"130.0\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_9_16\" x=\"136.625\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_10_16\" x=\"143.25\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_11_16\" x=\"149.875\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_12_16\" x=\"156.5\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_13_16\" x=\"163.125\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_14_16\" x=\"169.75\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_15_16\" x=\"176.375\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"mem_tile_memory4\" class=\"mem_tile_memory\" width=\"106\" height=\"46.0\" x=\"77\" y=\"737.0\" stroke=\"black\" stroke-width=\"\" fill=\"None\"/><animate attributeName=\"stroke-width\" dur=\"0.5s\" repeatCount=\"indefinite\" values=\"0;                       2;0\"/>\n",
       " </g>\n",
       "<g> <text id=\"text_mem_tile_memory4\" class=\"mem_tile_memory_text\" x=\"77\" y=\"737.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"750.9\" dy=\"1em\" font-size=\"13\"> </tspan></text></g>\n",
       "<g> <rect id=\"if_tile5\" class=\"if_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"820\" stroke=\"black\" stroke-width=\"2\" fill=\"#E0C2FF\"/> </g>\n",
       "<g> <text id=\"text_if_tile5\" class=\"if_tile_text\" x=\"20\" y=\"820\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"889.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box5\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"830\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box5\" class=\"interconnect_box_text\" x=\"30\" y=\"830\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"849.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line5_0\" class=\"interconnect\" x1=\"38.0\" y1=\"845.0\" x2=\"58.0\" y2=\"845.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line5_1\" class=\"interconnect\" x1=\"62.0\" y1=\"855.0\" x2=\"42.0\" y2=\"855.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line5_2\" class=\"interconnect\" x1=\"45.0\" y1=\"862.0\" x2=\"45.0\" y2=\"842.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line5_3\" class=\"interconnect\" x1=\"55.0\" y1=\"838.0\" x2=\"55.0\" y2=\"858.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"if_tile_container5\" class=\"if_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"890.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_if_tile_container5\" class=\"if_tile_container_text\" x=\"70\" y=\"890.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"919.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"if_dmabox5\" class=\"if_dmabox\" width=\"106\" height=\"46.0\" x=\"77\" y=\"897.0\" stroke=\"black\" stroke-width=\"\" fill=\"white\"/><animate attributeName=\"stroke-width\" dur=\"0.5s\" repeatCount=\"indefinite\" values=\"0;                       2;0\"/>\n",
       " </g>\n",
       "<g> <text id=\"text_if_dmabox5\" class=\"if_dmabox_text\" x=\"77\" y=\"897.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"92.0\" y=\"904.4\" dy=\"1em\" font-size=\"13\">Interface Tile </tspan><tspan x=\"93.0\" y=\"917.4\" dy=\"1em\" font-size=\"13\">Data movers </tspan></text></g>\n",
       "<line id=\"diagonal5\" class=\"interconnect\" x1=\"70\" y1=\"870\" x2=\"90.0\" y2=\"890.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<path id=\"south_mem_connection5\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" d=\"M 130.0, 950.0,                 l0, 30\"/>\n",
       "<circle id=\"d_circle5\" class=\"ic_animations\" cx=\"90.0\" cy=\"890.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#009E73\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle5\" class=\"ic_animations\" cx=\"70\" cy=\"870\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle5\" class=\"ic_animations\" cx=\"70\" cy=\"870\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"mem_circle_east5\" class=\"mem_animations\" cx=\"0\" cy=\"0\" r=\"3\" fill=\"none\">                    <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" path=\"M 130.0, 980,                         l 0, -30                        \" begin=\"0s\"/>                    <animate attributeName=\"fill\" values=\"none;#009E73\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>                    </circle>\n",
       "<circle id=\"mem_circle_east5\" class=\"mem_animations\" cx=\"0\" cy=\"0\" r=\"3\" fill=\"none\">                    <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" path=\"M 130.0, 950,                         l 0, 30                        \" begin=\"0s\"/>                    <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>                    </circle>\n",
       "<circle id=\"mem_circle_east5\" class=\"mem_animations\" cx=\"0\" cy=\"0\" r=\"3\" fill=\"none\">                    <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" path=\"M 130.0, 950,                         l 0, 30                        \" begin=\"0s\"/>                    <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>                    </circle>\n",
       "<line id=\"diagonal0\" class=\"interconnect\" x1=\"70\" y1=\"70\" x2=\"90.0\" y2=\"90.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection0\" class=\"interconnect\" x1=\"50\" y1=\"70\" x2=\"50\" y2=\"190\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal1\" class=\"interconnect\" x1=\"70\" y1=\"230\" x2=\"90.0\" y2=\"250.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection1\" class=\"interconnect\" x1=\"50\" y1=\"230\" x2=\"50\" y2=\"350\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal2\" class=\"interconnect\" x1=\"70\" y1=\"390\" x2=\"90.0\" y2=\"410.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection2\" class=\"interconnect\" x1=\"50\" y1=\"390\" x2=\"50\" y2=\"510\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal3\" class=\"interconnect\" x1=\"70\" y1=\"550\" x2=\"90.0\" y2=\"570.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection3\" class=\"interconnect\" x1=\"50\" y1=\"550\" x2=\"50\" y2=\"670\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal4\" class=\"interconnect\" x1=\"70\" y1=\"710\" x2=\"90.0\" y2=\"730.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection4\" class=\"interconnect\" x1=\"50\" y1=\"710\" x2=\"50\" y2=\"830\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<circle id=\"d_circle3\" class=\"ic_animations\" cx=\"70\" cy=\"550\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"-0.2s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"-0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north3\" class=\"ic_animations\" cx=\"50\" cy=\"670\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"-0.2s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"-0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle3\" class=\"ic_animations\" cx=\"90.0\" cy=\"570.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south3\" class=\"ic_animations\" cx=\"50\" cy=\"550\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"70\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#009E73\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north4\" class=\"ic_animations\" cx=\"50\" cy=\"830\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#009E73\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"90.0\" cy=\"730.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"-0.2s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"-0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"90.0\" cy=\"730.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south4\" class=\"ic_animations\" cx=\"50\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"90.0\" cy=\"730.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south4\" class=\"ic_animations\" cx=\"50\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"70\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<path id=\"south_mem_connection0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 160.0, 143.0,                 l0, 80.0,                l-30.0, 17.0,                l-30.0, 17.0\"/>\n",
       "<path id=\"south_mem_connection0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 100.0, 143.0,                 l0, 80.0,                l30.0, 17.0,                l30.0, 17.0\"/>\n",
       "<line id=\"internal_mem_connection0\" x1=\"123.0\" y1=\"120.0\" x2=\"137.0\" y2=\"120.0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<path id=\"south_mem_connection1\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 160.0, 303.0,                 l0, 80.0,                l-30.0, 17.0,                l-30.0, 17.0\"/>\n",
       "<path id=\"south_mem_connection1\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 100.0, 303.0,                 l0, 80.0,                l30.0, 17.0,                l30.0, 17.0\"/>\n",
       "<line id=\"internal_mem_connection1\" x1=\"123.0\" y1=\"280.0\" x2=\"137.0\" y2=\"280.0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<path id=\"south_mem_connection2\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 160.0, 463.0,                 l0, 80.0,                l-30.0, 17.0,                l-30.0, 17.0\"/>\n",
       "<path id=\"south_mem_connection2\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 100.0, 463.0,                 l0, 80.0,                l30.0, 17.0,                l30.0, 17.0\"/>\n",
       "<line id=\"internal_mem_connection2\" x1=\"123.0\" y1=\"440.0\" x2=\"137.0\" y2=\"440.0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<linearGradient id=\"kernel_3\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#C00000\"/> </linearGradient> <circle id=\"kernel_circle_3\" class=\"kernel\" cx=\"100.0\" cy=\"600.0\" r=\"20\" fill=\"url(#kernel_3)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 100.0 600.0\" to=\"360 100.0 600.0\" dur=\"4s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<g transform=\"rotate(180, 139.875, 623.0) \"> <rect id=\"buffer3_0\" class=\"aie_tile_buffers\" x=\"137.0\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C36D6D;#C36D6D;#C36D6D;#C36D6D\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 145.625, 623.0) \"> <rect id=\"buffer3_1\" class=\"aie_tile_buffers\" x=\"142.75\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C36D6D;#C36D6D;#C36D6D;#C36D6D\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 151.375, 623.0) \"> <rect id=\"buffer3_2\" class=\"aie_tile_buffers\" x=\"148.5\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C00000;#C00000;#C00000;#C00000\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 157.125, 623.0) \"> <rect id=\"buffer3_3\" class=\"aie_tile_buffers\" x=\"154.25\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C00000;#C00000;#C00000;#C00000\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 80.3125, 783.0) \"> <rect id=\"buffer4_0\" class=\"mem_tile_buffers\" x=\"77.0\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C36D6D;#C36D6D;#009E73;#009E73\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 86.9375, 783.0) \"> <rect id=\"buffer4_1\" class=\"mem_tile_buffers\" x=\"83.625\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#009E73;#009E73;#C36D6D;#C36D6D\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 93.5625, 783.0) \"> <rect id=\"buffer4_2\" class=\"mem_tile_buffers\" x=\"90.25\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#4B0092;#4B0092;#C00000;#C00000\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 100.1875, 783.0) \"> <rect id=\"buffer4_3\" class=\"mem_tile_buffers\" x=\"96.875\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C00000;#C00000;#4B0092;#4B0092\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<linearGradient id=\"kernel_1001\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#C00000\"/> </linearGradient> <circle id=\"kernel_circle_1001\" class=\"kernel\" cx=\"260.0\" cy=\"940.0\" r=\"20\" fill=\"url(#kernel_1001)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 260.0 940.0\" to=\"360 260.0 940.0\" dur=\"0s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<text class=\"text\" x=\"288\" y=\"945.0\">rgba_rtp_thresh_0</text></g></svg>"
      ],
      "text/plain": [
       "<IPython.core.display.SVG object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "app_builder.display()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "271a0c74-668e-4d62-855c-6bee4d50450e",
   "metadata": {},
   "source": [
    "### Understanding data movement in the design\n",
    "\n",
    "We will start by following the movement of data in the NPU column. Remember that this is just one implementation of the application. The same design could map the software kernel to a different compute tile. The data movement would look different for another implementation.\n",
    "\n",
    "* External input data:\n",
    "\n",
    "  Notice the **green** token representing the input data. It moves from external memory (not shown) via the interface tile to the memory tile.\n",
    "  The input data fills a data buffer in the memory tile data memory.\n",
    "\n",
    "* Input to the compute tile:\n",
    "\n",
    "  The data buffer in the memory tile is shown as **green** while it fills. This represents the data being written to the buffer from the interface tile. Notice that once the data buffer fills, the color of the buffer changes to **pink** as it empties. This represents that the input data can be partitioned, segmented or reorganized in some way before it is sent to the compute tile.\n",
    "\n",
    "* Compute tile computation\n",
    "\n",
    "  Data moves to the compute tile and fills a data buffer in its data memory. Once one buffer is full it is released by the memory tile data movers, which lock the next buffer and continuing writing input data. The buffer that is full and has just been released can be used by the compute tile. It reads the data in this buffer and generates its output to a new buffer, shown in **red**. It will also write to a buffer until full, then switch to another buffer to continue writing output data.\n",
    "\n",
    "* Compute tile output data\n",
    "\n",
    "  The output data from the compute tile is moved back to the memory tile. The data buffers are shown in **red** as they fill indicating that the data being written from the compute tile. The buffers change color to **purple** to indicate the data changes ownership before being sent from the memory tile to the interface tile and back to external memory. \n",
    "     "
   ]
  },
  {
   "cell_type": "markdown",
   "id": "d7e7ccda",
   "metadata": {},
   "source": [
    "The memory tile contains 512KB local data memory and six data movers in each direction. Input data can be streamed into the data memory of the memory tile where it can be partitioned, segmented or reorganized in some way before it is streamed to one or more compute tiles. Most of the applications we have looked at so far read a stream of pixels, process them one by one and produce an output stream of data. The memory tile allows higher levels of data reuse, increasing the efficiency of the NPU. \n",
    "\n",
    "Returning to the code, `MTPassThrough()` is instantiate with no arguments in the `__init__` method and used in the call graph for the memory tile. \n",
    "\n",
    "```python\n",
    "        rows = x_in.shape[0]\n",
    "        bytes_per_row = x_in.shape[1] * x_in.shape[2]\n",
    "        for row in range(rows):\n",
    "            mtpin = self.mtbuffer_in(x_in[row])\n",
    "            x = self.k(mtbin, bytes_per_row, 128, 128, 128)\n",
    "            mtpout = self.mtbuffer_out(x)\n",
    "            x_out[row] = mtbout\n",
    "```\n",
    "\n",
    "In this example, `MTPassThrough()` only buffers data in the memory of the memory tile. It does not carry out any operation on the data. `self.mtbuffer_in()` buffers the input tile in the memory tile before sending it to the compute tile, the `self.mtbuffer_out()` buffers the output data in the memory tile before it is streamed out of the array. Note that, `mtpin` and `mtpout` are references to the buffers that are allocated in the memory tile.\n",
    "\n",
    "In the following notebook you will see how to multicast and broadcast using the memory tile. "
   ]
  },
  {
   "cell_type": "markdown",
   "id": "e6b60f93-e35d-4938-beb9-363e975e7c26",
   "metadata": {},
   "source": [
    "## Using the Memory Tile for Data Parallel Applications\n",
    "\n",
    "To scale this application for execution on all four compute tiles within an NPU column, we will use the memory tile. This involves dividing the input data into four segments and distributing one segment to each of the four compute tiles.\n",
    "\n",
    "The interface tile only supports two data movers in each direction. It cannot stream data concurrently to more than two compute tiles simultaneously. In this example the memory tile is used to partition the input data and stream data segments to each of the compute tiles.\n",
    "\n",
    "We will define the application and the example code to show:\n",
    "\n",
    "* Kernels can be instantiated multiple times\n",
    "* How `MTSplit()` can be used to split or divide data for streaming to different tiles\n",
    "* How `MTConcat()` can be used to concatenate data from different sources\n",
    "\n",
    "The full application code is as follows:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "5b4741f2-b7cd-4303-af27-bdf849b85a7e",
   "metadata": {
    "tags": []
   },
   "outputs": [],
   "source": [
    "from npu.build.mtkernel import MTSplit, MTConcat\n",
    "\n",
    "class ScaledUpThresholdApplication(AppBuilder):\n",
    "\n",
    "    def __init__(self):\n",
    "        self.threshold_kernels = [RgbaRtpThres() for _ in range(4)]\n",
    "        self.mtbsplit = MTSplit(4)\n",
    "        self.mtbconcat = MTConcat()\n",
    "        super().__init__()\n",
    "\n",
    "    def callgraph(self,x_in:np.ndarray, x_out:np.ndarray):\n",
    "\n",
    "        r_thresh = [ 51, 149, 128, 19  ]\n",
    "        g_thresh = [ 66, 12,  95,  128 ]\n",
    "        b_thresh = [ 0,  128, 17,  33  ]\n",
    "\n",
    "        kernel_outputs = [None] * 4\n",
    "\n",
    "        rows = x_in.shape[0]\n",
    "        pixels_per_row = x_in.shape[1]\n",
    "        for row in range(rows):\n",
    "            inputs = self.mtbsplit(x_in[row])\n",
    "            for i in range(4):\n",
    "                kernel_outputs[i] = self.threshold_kernels[i](inputs[i],\n",
    "                                    pixels_per_row,\n",
    "                                    r_thresh[i],\n",
    "                                    g_thresh[i],\n",
    "                                    b_thresh[i])\n",
    "            mtbuffer_out = self.mtbconcat(kernel_outputs)\n",
    "            x_out[row] = mtbuffer_out"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "169579bd-5345-4a6b-acbf-ef1df2f274a9",
   "metadata": {},
   "source": [
    "### Review the memory tile application code\n",
    "\n",
    "#### Import memory tile methods\n",
    "\n",
    "Notice the memory tile methods that are imported:\n",
    "\n",
    "```python\n",
    "from npu.build.mtkernel import MTSplit, MTConcat\n",
    "```\n",
    "\n",
    "#### Instantiate kernels\n",
    "\n",
    "In the `__init__` method, four instances of the threshold kernel are declared, using a [list comprehension](https://docs.python.org/3.10/tutorial/datastructures.html#list-comprehensions):\n",
    "\n",
    "```python\n",
    "    self.threshold_kernels  = [RgbaRtpThres() for _ in range(4)]\n",
    "```\n",
    "\n",
    "#### Threshold parameters\n",
    "\n",
    "There are four kernels, so we need four sets of RGB threshold values. We will declare lists with arbitrary initial values for each of the RGB parameters. \n",
    "\n",
    "```python\n",
    "    r_thresh = [ 51, 149, 128, 19  ]\n",
    "    g_thresh = [ 66, 12,  95,  128 ]\n",
    "    b_thresh = [ 0,  128, 17,  33  ]\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "5ff3acd4-6461-4b39-b0bd-be3256917122",
   "metadata": {},
   "source": [
    "### `MTSplit`\n",
    "\n",
    "In this example, we divide the input data into four separate buffer objects, and each of these is streamed to a compute tile. The threshold function operates on a pixel-by-pixel basis, which means there are no data dependencies between pixels. This allows us to split and send the data in any order and size, as long as we ensure that the returned data is reconstructed in the same order it was sent.\n",
    "\n",
    "Similar to previous examples, we follow the approach of streaming one row of the input image at a time and further dividing this row into four parts. We use the `MTSplit()` function to achieve this data splitting. The first step is to instantiate `MTSplit()` in the `__init__` method with the number of splits, in this case `4`. In the `callgraph`, we call `MTSplit()` which returns a list of separate buffer objects, which we assign to `inputs`. In this particular case, we split the input into four parts:\n",
    "\n",
    "```python\n",
    "    self.mtbsplit = MTSplit(4) # in __init__\n",
    "    ...\n",
    "    inputs = self.mtbsplit(x_in[row]) # in callgraph\n",
    "```\n",
    "\n",
    "Given that `input[t]` represents a row of 1,280 pixels, and each pixel is 4-Byte, each of the four returned objects will be 1280 * 4 / 4, which equals 1280-Byte each. This creates a tiling effect on the image, with each compute tile processing one quarter of the row. The first compute tile handles the first quarter, and so on. This pattern repeats for each row, resulting in a column effect when you run the application. \n",
    "\n",
    "\n",
    "### Connect the inputs to each kernel\n",
    "\n",
    "Inside the loop each kernel is assigned one of the inputs returned from `self.mtbsplit()` call. The object returned from each of the kernels represents the output data connection from that kernel, that is assigned to the `kernel_outputs` list.\n",
    "\n",
    "```python\n",
    "    kernel_outputs[i] = self.threshold_kernels [i](inputs[i], ...)\n",
    "```\n",
    "### Connect the output from each kernel\n",
    "\n",
    "We first instantiate `MTConcat()` in the `__init__` method, then we can call `self.mtbconcat()` which takes multiple input buffers and effectively concatenates them. The memory tile will write these buffers out as one data stream which will be the output of the application. This output is streamed to the interface tile and back to external memory.\n",
    "\n",
    "```python\n",
    "    self.mtbconcat = MTConcat() # in __init__\n",
    "    ...\n",
    "    mtbuffer_out = self.mtbconcat(kernel_outputs) # in callgraph\n",
    "```"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "a46cd7b5-9368-49ed-be28-f64467ee268c",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "image/svg+xml": [
       "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" id=\"svg\" version=\"1.1\" width=\"420\" height=\"980\" viewBox=\"0, 0, 420, 980\">\n",
       "            <defs>\n",
       "                <marker id=\"ic_startarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"10\" refY=\"3.5\" orient=\"auto\">\n",
       "                <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "                <marker id=\"ic_endarrow\" markerWidth=\"7\" markerHeight=\"4\" refX=\"0\" refY=\"2\" orient=\"auto\" markerUnits=\"strokeWidth\">\n",
       "                    <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "\n",
       "                <marker id=\"red_startarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"0\" refY=\"2\" orient=\"auto\">\n",
       "                <polygon points=\"0 2, 4 0, 4 4\" fill=\"red\"/>\n",
       "                </marker>\n",
       "                <marker id=\"red_endarrow\" markerWidth=\"7\" markerHeight=\"4\" refX=\"4\" refY=\"2\" orient=\"auto\" markerUnits=\"strokeWidth\">\n",
       "                    <polygon points=\"0 0, 4 2, 0 4\" fill=\"red\"/>\n",
       "                </marker>\n",
       "\n",
       "                <marker id=\"startarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"10\" refY=\"3.5\" orient=\"auto\">\n",
       "                <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "                <marker id=\"endarrow\" markerWidth=\"10\" markerHeight=\"7\" refX=\"0\" refY=\"3.5\" orient=\"auto\" markerUnits=\"strokeWidth\">\n",
       "                    <polygon points=\"0 0, 4 2, 0 4\" fill=\"black\"/>\n",
       "                </marker>\n",
       "            </defs>\n",
       "            <style>                .ic_animations {                     opacity: 0.8;                 }            </style>\n",
       "            <g id=\"svg\"><rect id=\"border\" class=\"border\" width=\"400\" height=\"960\" x=\"10.0\" y=\"10.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/>\n",
       "<g> <rect id=\"aie_tile0\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"20\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile0\" class=\"aie_tile_text\" x=\"20\" y=\"20\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"89.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box0\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"30\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box0\" class=\"interconnect_box_text\" x=\"30\" y=\"30\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"49.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line0_0\" class=\"interconnect\" x1=\"38.0\" y1=\"45.0\" x2=\"58.0\" y2=\"45.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line0_1\" class=\"interconnect\" x1=\"62.0\" y1=\"55.0\" x2=\"42.0\" y2=\"55.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line0_2\" class=\"interconnect\" x1=\"45.0\" y1=\"62.0\" x2=\"45.0\" y2=\"42.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line0_3\" class=\"interconnect\" x1=\"55.0\" y1=\"38.0\" x2=\"55.0\" y2=\"58.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container0\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"90.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container0\" class=\"aie_tile_container_text\" x=\"70\" y=\"90.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"119.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box0\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"97.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box0\" class=\"aie_box_text\" x=\"77.0\" y=\"97.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"100.0\" y=\"111.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"97.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box0\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"97.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box0\" class=\"local_memory_box_text\" x=\"137.0\" y=\"97.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"160.0\" y=\"111.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_tile1\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"180\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile1\" class=\"aie_tile_text\" x=\"20\" y=\"180\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"249.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box1\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"190\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box1\" class=\"interconnect_box_text\" x=\"30\" y=\"190\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"209.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line1_0\" class=\"interconnect\" x1=\"38.0\" y1=\"205.0\" x2=\"58.0\" y2=\"205.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line1_1\" class=\"interconnect\" x1=\"62.0\" y1=\"215.0\" x2=\"42.0\" y2=\"215.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line1_2\" class=\"interconnect\" x1=\"45.0\" y1=\"222.0\" x2=\"45.0\" y2=\"202.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line1_3\" class=\"interconnect\" x1=\"55.0\" y1=\"198.0\" x2=\"55.0\" y2=\"218.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container1\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"250.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container1\" class=\"aie_tile_container_text\" x=\"70\" y=\"250.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"279.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box1\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"257.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box1\" class=\"aie_box_text\" x=\"77.0\" y=\"257.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"100.0\" y=\"271.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"257.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box1\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"257.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box1\" class=\"local_memory_box_text\" x=\"137.0\" y=\"257.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"160.0\" y=\"271.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_tile2\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"340\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile2\" class=\"aie_tile_text\" x=\"20\" y=\"340\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"409.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box2\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"350\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box2\" class=\"interconnect_box_text\" x=\"30\" y=\"350\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"369.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line2_0\" class=\"interconnect\" x1=\"38.0\" y1=\"365.0\" x2=\"58.0\" y2=\"365.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line2_1\" class=\"interconnect\" x1=\"62.0\" y1=\"375.0\" x2=\"42.0\" y2=\"375.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line2_2\" class=\"interconnect\" x1=\"45.0\" y1=\"382.0\" x2=\"45.0\" y2=\"362.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line2_3\" class=\"interconnect\" x1=\"55.0\" y1=\"358.0\" x2=\"55.0\" y2=\"378.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container2\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"410.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container2\" class=\"aie_tile_container_text\" x=\"70\" y=\"410.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"439.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box2\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"417.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box2\" class=\"aie_box_text\" x=\"77.0\" y=\"417.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"100.0\" y=\"431.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"417.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box2\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"417.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box2\" class=\"local_memory_box_text\" x=\"137.0\" y=\"417.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"160.0\" y=\"431.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_tile3\" class=\"aie_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"500\" stroke=\"black\" stroke-width=\"2\" fill=\"#C2E9FF\"/> </g>\n",
       "<g> <text id=\"text_aie_tile3\" class=\"aie_tile_text\" x=\"20\" y=\"500\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"569.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box3\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"510\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box3\" class=\"interconnect_box_text\" x=\"30\" y=\"510\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"529.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line3_0\" class=\"interconnect\" x1=\"38.0\" y1=\"525.0\" x2=\"58.0\" y2=\"525.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line3_1\" class=\"interconnect\" x1=\"62.0\" y1=\"535.0\" x2=\"42.0\" y2=\"535.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line3_2\" class=\"interconnect\" x1=\"45.0\" y1=\"542.0\" x2=\"45.0\" y2=\"522.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line3_3\" class=\"interconnect\" x1=\"55.0\" y1=\"518.0\" x2=\"55.0\" y2=\"538.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"aie_tile_container3\" class=\"aie_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"570.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_aie_tile_container3\" class=\"aie_tile_container_text\" x=\"70\" y=\"570.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"599.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"aie_box3\" class=\"aie_box\" width=\"46.0\" height=\"46.0\" x=\"77.0\" y=\"577.0\" stroke=\"black\" stroke-width=\"1\" fill=\"white\"/> </g>\n",
       "<g> <text id=\"text_aie_box3\" class=\"aie_box_text\" x=\"77.0\" y=\"577.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"100.0\" y=\"591.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_0_8\" x=\"137.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_8\" x=\"142.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_8\" x=\"148.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_8\" x=\"154.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_8\" x=\"160.0\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_8\" x=\"165.75\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_8\" x=\"171.5\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_8\" x=\"177.25\" y=\"577.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"local_memory_box3\" class=\"local_memory_box\" width=\"46.0\" height=\"46.0\" x=\"137.0\" y=\"577.0\" stroke=\"black\" stroke-width=\"1\" fill=\"None\"/> </g>\n",
       "<g> <text id=\"text_local_memory_box3\" class=\"local_memory_box_text\" x=\"137.0\" y=\"577.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"160.0\" y=\"591.6\" dy=\"1em\" font-size=\"12\"> </tspan></text></g>\n",
       "<g> <rect id=\"mem_tile4\" class=\"mem_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"660\" stroke=\"black\" stroke-width=\"2\" fill=\"#E0FFC2\"/> </g>\n",
       "<g> <text id=\"text_mem_tile4\" class=\"mem_tile_text\" x=\"20\" y=\"660\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"729.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box4\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"670\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box4\" class=\"interconnect_box_text\" x=\"30\" y=\"670\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"689.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line4_0\" class=\"interconnect\" x1=\"38.0\" y1=\"685.0\" x2=\"58.0\" y2=\"685.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line4_1\" class=\"interconnect\" x1=\"62.0\" y1=\"695.0\" x2=\"42.0\" y2=\"695.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line4_2\" class=\"interconnect\" x1=\"45.0\" y1=\"702.0\" x2=\"45.0\" y2=\"682.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line4_3\" class=\"interconnect\" x1=\"55.0\" y1=\"678.0\" x2=\"55.0\" y2=\"698.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"mem_tile_container4\" class=\"mem_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"730.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_mem_tile_container4\" class=\"mem_tile_container_text\" x=\"70\" y=\"730.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"759.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"memory_outline_0_16\" x=\"77.0\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_1_16\" x=\"83.625\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_2_16\" x=\"90.25\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_3_16\" x=\"96.875\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_4_16\" x=\"103.5\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_5_16\" x=\"110.125\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_6_16\" x=\"116.75\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_7_16\" x=\"123.375\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_8_16\" x=\"130.0\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_9_16\" x=\"136.625\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_10_16\" x=\"143.25\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_11_16\" x=\"149.875\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_12_16\" x=\"156.5\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_13_16\" x=\"163.125\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_14_16\" x=\"169.75\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"memory_outline_15_16\" x=\"176.375\" y=\"737.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:#f0f0f0; fill:#fefefe; stroke-opacity:0.5\"> </rect> </g>\n",
       "<g> <rect id=\"mem_tile_memory4\" class=\"mem_tile_memory\" width=\"106\" height=\"46.0\" x=\"77\" y=\"737.0\" stroke=\"black\" stroke-width=\"\" fill=\"None\"/><animate attributeName=\"stroke-width\" dur=\"0.5s\" repeatCount=\"indefinite\" values=\"0;                       2;0\"/>\n",
       " </g>\n",
       "<g> <text id=\"text_mem_tile_memory4\" class=\"mem_tile_memory_text\" x=\"77\" y=\"737.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"750.9\" dy=\"1em\" font-size=\"13\"> </tspan></text></g>\n",
       "<g> <rect id=\"if_tile5\" class=\"if_tile\" width=\"180\" height=\"140\" x=\"20\" y=\"820\" stroke=\"black\" stroke-width=\"2\" fill=\"#E0C2FF\"/> </g>\n",
       "<g> <text id=\"text_if_tile5\" class=\"if_tile_text\" x=\"20\" y=\"820\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"110.0\" y=\"889.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"interconnect_box5\" class=\"interconnect_box\" width=\"40\" height=\"40\" x=\"30\" y=\"830\" stroke=\"black\" stroke-width=\"1\" fill=\"#DBBCBD\"/> </g>\n",
       "<g> <text id=\"text_interconnect_box5\" class=\"interconnect_box_text\" x=\"30\" y=\"830\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"50.0\" y=\"849.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<line id=\"ic_h_line5_0\" class=\"interconnect\" x1=\"38.0\" y1=\"845.0\" x2=\"58.0\" y2=\"845.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line5_1\" class=\"interconnect\" x1=\"62.0\" y1=\"855.0\" x2=\"42.0\" y2=\"855.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line5_2\" class=\"interconnect\" x1=\"45.0\" y1=\"862.0\" x2=\"45.0\" y2=\"842.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<line id=\"ic_h_line5_3\" class=\"interconnect\" x1=\"55.0\" y1=\"838.0\" x2=\"55.0\" y2=\"858.0\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" marker-end=\"url(#ic_endarrow)\"/>\n",
       "<g> <rect id=\"if_tile_container5\" class=\"if_tile_container\" width=\"120\" height=\"60.0\" x=\"70\" y=\"890.0\" stroke=\"black\" stroke-width=\"1\" fill=\"#ffffff\"/> </g>\n",
       "<g> <text id=\"text_if_tile_container5\" class=\"if_tile_container_text\" x=\"70\" y=\"890.0\" fill=\"\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"130.0\" y=\"919.3\" dy=\"1em\" font-size=\"1\"> </tspan></text></g>\n",
       "<g> <rect id=\"if_dmabox5\" class=\"if_dmabox\" width=\"106\" height=\"46.0\" x=\"77\" y=\"897.0\" stroke=\"black\" stroke-width=\"\" fill=\"white\"/><animate attributeName=\"stroke-width\" dur=\"0.5s\" repeatCount=\"indefinite\" values=\"0;                       2;0\"/>\n",
       " </g>\n",
       "<g> <text id=\"text_if_dmabox5\" class=\"if_dmabox_text\" x=\"77\" y=\"897.0\" fill=\"black\" font-family=\"Arial, Helvetica, sans-serif\"> <tspan x=\"92.0\" y=\"904.4\" dy=\"1em\" font-size=\"13\">Interface Tile </tspan><tspan x=\"93.0\" y=\"917.4\" dy=\"1em\" font-size=\"13\">Data movers </tspan></text></g>\n",
       "<line id=\"diagonal5\" class=\"interconnect\" x1=\"70\" y1=\"870\" x2=\"90.0\" y2=\"890.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<path id=\"south_mem_connection5\" stroke=\"black\" stroke-width=\"1\" fill=\"none\" d=\"M 130.0, 950.0,                 l0, 30\"/>\n",
       "<circle id=\"d_circle5\" class=\"ic_animations\" cx=\"90.0\" cy=\"890.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#009E73\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle5\" class=\"ic_animations\" cx=\"70\" cy=\"870\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle5\" class=\"ic_animations\" cx=\"70\" cy=\"870\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"mem_circle_east5\" class=\"mem_animations\" cx=\"0\" cy=\"0\" r=\"3\" fill=\"none\">                    <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" path=\"M 130.0, 980,                         l 0, -30                        \" begin=\"0s\"/>                    <animate attributeName=\"fill\" values=\"none;#009E73\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>                    </circle>\n",
       "<circle id=\"mem_circle_east5\" class=\"mem_animations\" cx=\"0\" cy=\"0\" r=\"3\" fill=\"none\">                    <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" path=\"M 130.0, 950,                         l 0, 30                        \" begin=\"0s\"/>                    <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>                    </circle>\n",
       "<circle id=\"mem_circle_east5\" class=\"mem_animations\" cx=\"0\" cy=\"0\" r=\"3\" fill=\"none\">                    <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" path=\"M 130.0, 950,                         l 0, 30                        \" begin=\"0s\"/>                    <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>                    </circle>\n",
       "<line id=\"diagonal0\" class=\"interconnect\" x1=\"70\" y1=\"70\" x2=\"90.0\" y2=\"90.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection0\" class=\"interconnect\" x1=\"50\" y1=\"70\" x2=\"50\" y2=\"190\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal1\" class=\"interconnect\" x1=\"70\" y1=\"230\" x2=\"90.0\" y2=\"250.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection1\" class=\"interconnect\" x1=\"50\" y1=\"230\" x2=\"50\" y2=\"350\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal2\" class=\"interconnect\" x1=\"70\" y1=\"390\" x2=\"90.0\" y2=\"410.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection2\" class=\"interconnect\" x1=\"50\" y1=\"390\" x2=\"50\" y2=\"510\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal3\" class=\"interconnect\" x1=\"70\" y1=\"550\" x2=\"90.0\" y2=\"570.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection3\" class=\"interconnect\" x1=\"50\" y1=\"550\" x2=\"50\" y2=\"670\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"diagonal4\" class=\"interconnect\" x1=\"70\" y1=\"710\" x2=\"90.0\" y2=\"730.0\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<line id=\"v_ic_connection4\" class=\"interconnect\" x1=\"50\" y1=\"710\" x2=\"50\" y2=\"830\" stroke=\"red\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<circle id=\"d_circle0\" class=\"ic_animations\" cx=\"70\" cy=\"70\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#E6BBD1\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north0\" class=\"ic_animations\" cx=\"50\" cy=\"190\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#E6BBD1\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle0\" class=\"ic_animations\" cx=\"90.0\" cy=\"90.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.6s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#DC267F\" dur=\"0.0001s\" begin=\"0.6s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south0\" class=\"ic_animations\" cx=\"50\" cy=\"70\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.6s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#DC267F\" dur=\"0.0001s\" begin=\"0.6s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle1\" class=\"ic_animations\" cx=\"70\" cy=\"230\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.2s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#E69F00\" dur=\"0.0001s\" begin=\"0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north1\" class=\"ic_animations\" cx=\"50\" cy=\"350\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.2s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#E69F00\" dur=\"0.0001s\" begin=\"0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north1\" class=\"ic_animations\" cx=\"50\" cy=\"350\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#E6BBD1\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle1\" class=\"ic_animations\" cx=\"90.0\" cy=\"250.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#F0E442\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south1\" class=\"ic_animations\" cx=\"50\" cy=\"230\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#F0E442\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south1\" class=\"ic_animations\" cx=\"50\" cy=\"230\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.6s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#DC267F\" dur=\"0.0001s\" begin=\"0.6s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle2\" class=\"ic_animations\" cx=\"70\" cy=\"390\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#B2D4E8\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north2\" class=\"ic_animations\" cx=\"50\" cy=\"510\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#B2D4E8\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north2\" class=\"ic_animations\" cx=\"50\" cy=\"510\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.2s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#E69F00\" dur=\"0.0001s\" begin=\"0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north2\" class=\"ic_animations\" cx=\"50\" cy=\"510\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#E6BBD1\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle2\" class=\"ic_animations\" cx=\"90.0\" cy=\"410.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.2s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#0072B2\" dur=\"0.0001s\" begin=\"0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south2\" class=\"ic_animations\" cx=\"50\" cy=\"390\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.2s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#0072B2\" dur=\"0.0001s\" begin=\"0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south2\" class=\"ic_animations\" cx=\"50\" cy=\"390\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#F0E442\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south2\" class=\"ic_animations\" cx=\"50\" cy=\"390\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.6s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#DC267F\" dur=\"0.0001s\" begin=\"0.6s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle3\" class=\"ic_animations\" cx=\"70\" cy=\"550\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"-0.2s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"-0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north3\" class=\"ic_animations\" cx=\"50\" cy=\"670\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"-0.2s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"-0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north3\" class=\"ic_animations\" cx=\"50\" cy=\"670\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#B2D4E8\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north3\" class=\"ic_animations\" cx=\"50\" cy=\"670\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.2s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#E69F00\" dur=\"0.0001s\" begin=\"0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north3\" class=\"ic_animations\" cx=\"50\" cy=\"670\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#E6BBD1\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle3\" class=\"ic_animations\" cx=\"90.0\" cy=\"570.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south3\" class=\"ic_animations\" cx=\"50\" cy=\"550\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south3\" class=\"ic_animations\" cx=\"50\" cy=\"550\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.2s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#0072B2\" dur=\"0.0001s\" begin=\"0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south3\" class=\"ic_animations\" cx=\"50\" cy=\"550\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#F0E442\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south3\" class=\"ic_animations\" cx=\"50\" cy=\"550\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.6s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#DC267F\" dur=\"0.0001s\" begin=\"0.6s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"70\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#009E73\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_north4\" class=\"ic_animations\" cx=\"50\" cy=\"830\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, -120                \"/>            <animate attributeName=\"fill\" values=\"none;#009E73\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"90.0\" cy=\"730.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"-0.2s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C36D6D\" dur=\"0.0001s\" begin=\"-0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"90.0\" cy=\"730.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#B2D4E8\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"90.0\" cy=\"730.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.2s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#E69F00\" dur=\"0.0001s\" begin=\"0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"90.0\" cy=\"730.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#E6BBD1\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"90.0\" cy=\"730.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south4\" class=\"ic_animations\" cx=\"50\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"90.0\" cy=\"730.0\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L-20.0, -20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"v_circle_south4\" class=\"ic_animations\" cx=\"50\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"4s\" repeatCount=\"indefinite\" begin=\"0s\" path=\"M0, 0,                 L0, 120                \"/>            <animate attributeName=\"fill\" values=\"none;#4B0092\" dur=\"0.0001s\" begin=\"0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"70\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.0s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#C00000\" dur=\"0.0001s\" begin=\"0.0s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"70\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.2s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#0072B2\" dur=\"0.0001s\" begin=\"0.2s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"70\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.4s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#F0E442\" dur=\"0.0001s\" begin=\"0.4s\" fill=\"freeze\"/>            </circle>\n",
       "<circle id=\"d_circle4\" class=\"ic_animations\" cx=\"70\" cy=\"710\" r=\"3\" fill=\"none\">            <animateMotion dur=\"2.0s\" repeatCount=\"indefinite\" begin=\"0.6s\" path=\"M0, 0,                 L20.0, 20.0,                \"/>            <animate attributeName=\"fill\" values=\"none;#DC267F\" dur=\"0.0001s\" begin=\"0.6s\" fill=\"freeze\"/>            </circle>\n",
       "<path id=\"south_mem_connection0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 160.0, 143.0,                 l0, 80.0,                l-30.0, 17.0,                l-30.0, 17.0\"/>\n",
       "<path id=\"south_mem_connection0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 100.0, 143.0,                 l0, 80.0,                l30.0, 17.0,                l30.0, 17.0\"/>\n",
       "<line id=\"internal_mem_connection0\" x1=\"123.0\" y1=\"120.0\" x2=\"137.0\" y2=\"120.0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<path id=\"south_mem_connection1\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 160.0, 303.0,                 l0, 80.0,                l-30.0, 17.0,                l-30.0, 17.0\"/>\n",
       "<path id=\"south_mem_connection1\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 100.0, 303.0,                 l0, 80.0,                l30.0, 17.0,                l30.0, 17.0\"/>\n",
       "<line id=\"internal_mem_connection1\" x1=\"123.0\" y1=\"280.0\" x2=\"137.0\" y2=\"280.0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<path id=\"south_mem_connection2\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 160.0, 463.0,                 l0, 80.0,                l-30.0, 17.0,                l-30.0, 17.0\"/>\n",
       "<path id=\"south_mem_connection2\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\" d=\"M 100.0, 463.0,                 l0, 80.0,                l30.0, 17.0,                l30.0, 17.0\"/>\n",
       "<line id=\"internal_mem_connection2\" x1=\"123.0\" y1=\"440.0\" x2=\"137.0\" y2=\"440.0\" class=\"mem_connections\" stroke=\"blue\" stroke-width=\"1\" fill=\"none\"/>\n",
       "<linearGradient id=\"kernel_0\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#DC267F\"/> </linearGradient> <circle id=\"kernel_circle_0\" class=\"kernel\" cx=\"100.0\" cy=\"120.0\" r=\"20\" fill=\"url(#kernel_0)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 100.0 120.0\" to=\"360 100.0 120.0\" dur=\"4s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<linearGradient id=\"kernel_1\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#F0E442\"/> </linearGradient> <circle id=\"kernel_circle_1\" class=\"kernel\" cx=\"100.0\" cy=\"280.0\" r=\"20\" fill=\"url(#kernel_1)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 100.0 280.0\" to=\"360 100.0 280.0\" dur=\"4s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<linearGradient id=\"kernel_2\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#0072B2\"/> </linearGradient> <circle id=\"kernel_circle_2\" class=\"kernel\" cx=\"100.0\" cy=\"440.0\" r=\"20\" fill=\"url(#kernel_2)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 100.0 440.0\" to=\"360 100.0 440.0\" dur=\"4s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<linearGradient id=\"kernel_3\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#C00000\"/> </linearGradient> <circle id=\"kernel_circle_3\" class=\"kernel\" cx=\"100.0\" cy=\"600.0\" r=\"20\" fill=\"url(#kernel_3)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 100.0 600.0\" to=\"360 100.0 600.0\" dur=\"4s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<g transform=\"rotate(180, 139.875, 143.0) \"> <rect id=\"buffer0_0\" class=\"aie_tile_buffers\" x=\"137.0\" y=\"143.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#E6BBD1;#E6BBD1;#E6BBD1;#E6BBD1\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 145.625, 143.0) \"> <rect id=\"buffer0_1\" class=\"aie_tile_buffers\" x=\"142.75\" y=\"143.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#E6BBD1;#E6BBD1;#E6BBD1;#E6BBD1\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 151.375, 143.0) \"> <rect id=\"buffer0_2\" class=\"aie_tile_buffers\" x=\"148.5\" y=\"143.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#DC267F;#DC267F;#DC267F;#DC267F\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 157.125, 143.0) \"> <rect id=\"buffer0_3\" class=\"aie_tile_buffers\" x=\"154.25\" y=\"143.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#DC267F;#DC267F;#DC267F;#DC267F\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 139.875, 303.0) \"> <rect id=\"buffer1_0\" class=\"aie_tile_buffers\" x=\"137.0\" y=\"303.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#E69F00;#E69F00;#E69F00;#E69F00\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 145.625, 303.0) \"> <rect id=\"buffer1_1\" class=\"aie_tile_buffers\" x=\"142.75\" y=\"303.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#E69F00;#E69F00;#E69F00;#E69F00\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 151.375, 303.0) \"> <rect id=\"buffer1_2\" class=\"aie_tile_buffers\" x=\"148.5\" y=\"303.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#F0E442;#F0E442;#F0E442;#F0E442\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 157.125, 303.0) \"> <rect id=\"buffer1_3\" class=\"aie_tile_buffers\" x=\"154.25\" y=\"303.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#F0E442;#F0E442;#F0E442;#F0E442\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 139.875, 463.0) \"> <rect id=\"buffer2_0\" class=\"aie_tile_buffers\" x=\"137.0\" y=\"463.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#B2D4E8;#B2D4E8;#B2D4E8;#B2D4E8\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 145.625, 463.0) \"> <rect id=\"buffer2_1\" class=\"aie_tile_buffers\" x=\"142.75\" y=\"463.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#B2D4E8;#B2D4E8;#B2D4E8;#B2D4E8\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 151.375, 463.0) \"> <rect id=\"buffer2_2\" class=\"aie_tile_buffers\" x=\"148.5\" y=\"463.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#0072B2;#0072B2;#0072B2;#0072B2\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 157.125, 463.0) \"> <rect id=\"buffer2_3\" class=\"aie_tile_buffers\" x=\"154.25\" y=\"463.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#0072B2;#0072B2;#0072B2;#0072B2\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 139.875, 623.0) \"> <rect id=\"buffer3_0\" class=\"aie_tile_buffers\" x=\"137.0\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C36D6D;#C36D6D;#C36D6D;#C36D6D\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 145.625, 623.0) \"> <rect id=\"buffer3_1\" class=\"aie_tile_buffers\" x=\"142.75\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C36D6D;#C36D6D;#C36D6D;#C36D6D\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 151.375, 623.0) \"> <rect id=\"buffer3_2\" class=\"aie_tile_buffers\" x=\"148.5\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C00000;#C00000;#C00000;#C00000\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 157.125, 623.0) \"> <rect id=\"buffer3_3\" class=\"aie_tile_buffers\" x=\"154.25\" y=\"623.0\" height=\"46.0\" width=\"5.75\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C00000;#C00000;#C00000;#C00000\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 80.3125, 783.0) \"> <rect id=\"buffer4_0\" class=\"mem_tile_buffers\" x=\"77.0\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C36D6D;#C36D6D;#009E73;#009E73\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 86.9375, 783.0) \"> <rect id=\"buffer4_1\" class=\"mem_tile_buffers\" x=\"83.625\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#B2D4E8;#B2D4E8;#009E73;#009E73\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 93.5625, 783.0) \"> <rect id=\"buffer4_2\" class=\"mem_tile_buffers\" x=\"90.25\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.4s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#E69F00;#E69F00;#009E73;#009E73\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.4s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 100.1875, 783.0) \"> <rect id=\"buffer4_3\" class=\"mem_tile_buffers\" x=\"96.875\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.6s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#E6BBD1;#E6BBD1;#009E73;#009E73\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.6s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 106.8125, 783.0) \"> <rect id=\"buffer4_4\" class=\"mem_tile_buffers\" x=\"103.5\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#009E73;#009E73;#C36D6D;#C36D6D\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 113.4375, 783.0) \"> <rect id=\"buffer4_5\" class=\"mem_tile_buffers\" x=\"110.125\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#009E73;#009E73;#B2D4E8;#B2D4E8\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 120.0625, 783.0) \"> <rect id=\"buffer4_6\" class=\"mem_tile_buffers\" x=\"116.75\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.4s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#009E73;#009E73;#E69F00;#E69F00\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.4s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 126.6875, 783.0) \"> <rect id=\"buffer4_7\" class=\"mem_tile_buffers\" x=\"123.375\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.6s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#009E73;#009E73;#E6BBD1;#E6BBD1\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.6s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 133.3125, 783.0) \"> <rect id=\"buffer4_8\" class=\"mem_tile_buffers\" x=\"130.0\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#4B0092;#4B0092;#C00000;#C00000\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 139.9375, 783.0) \"> <rect id=\"buffer4_9\" class=\"mem_tile_buffers\" x=\"136.625\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#4B0092;#4B0092;#0072B2;#0072B2\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 146.5625, 783.0) \"> <rect id=\"buffer4_10\" class=\"mem_tile_buffers\" x=\"143.25\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.4s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#4B0092;#4B0092;#F0E442;#F0E442\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.4s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 153.1875, 783.0) \"> <rect id=\"buffer4_11\" class=\"mem_tile_buffers\" x=\"149.875\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"46.0;0;46.0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.6s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#4B0092;#4B0092;#DC267F;#DC267F\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.6s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 159.8125, 783.0) \"> <rect id=\"buffer4_12\" class=\"mem_tile_buffers\" x=\"156.5\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"0.8s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#C00000;#C00000;#4B0092;#4B0092\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"0.8s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 166.4375, 783.0) \"> <rect id=\"buffer4_13\" class=\"mem_tile_buffers\" x=\"163.125\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"1.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#0072B2;#0072B2;#4B0092;#4B0092\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"1.0s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 173.0625, 783.0) \"> <rect id=\"buffer4_14\" class=\"mem_tile_buffers\" x=\"169.75\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"1.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#F0E442;#F0E442;#4B0092;#4B0092\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"1.2s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<g transform=\"rotate(180, 179.6875, 783.0) \"> <rect id=\"buffer4_15\" class=\"mem_tile_buffers\" x=\"176.375\" y=\"783.0\" height=\"46.0\" width=\"6.625\" style=\"stroke:none; fill:none\"> <animate attributeName=\"height\" values=\"0;46.0;0\" keyTimes=\"0;0.5;1\" dur=\"4.0s\" begin=\"1.4s\" fill=\"freeze\" repeatCount=\"indefinite\"/> <animate attributeName=\"fill\" values=\"none;#DC267F;#DC267F;#4B0092;#4B0092\" keyTimes=\"0; 0.00001; 0.5; 0.50001; 1\" dur=\"4.0s\" begin=\"1.4s\" fill=\"freeze\" repeatCount=\"indefinite\"/></rect></g>\n",
       "<linearGradient id=\"kernel_1001\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#C00000\"/> </linearGradient> <circle id=\"kernel_circle_1001\" class=\"kernel\" cx=\"260.0\" cy=\"940.0\" r=\"20\" fill=\"url(#kernel_1001)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 260.0 940.0\" to=\"360 260.0 940.0\" dur=\"0s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<text class=\"text\" x=\"288\" y=\"945.0\">rgba_rtp_thresh_0</text><linearGradient id=\"kernel_1002\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#0072B2\"/> </linearGradient> <circle id=\"kernel_circle_1002\" class=\"kernel\" cx=\"260.0\" cy=\"890.0\" r=\"20\" fill=\"url(#kernel_1002)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 260.0 890.0\" to=\"360 260.0 890.0\" dur=\"0s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<text class=\"text\" x=\"288\" y=\"895.0\">rgba_rtp_thresh_1</text><linearGradient id=\"kernel_1003\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#F0E442\"/> </linearGradient> <circle id=\"kernel_circle_1003\" class=\"kernel\" cx=\"260.0\" cy=\"840.0\" r=\"20\" fill=\"url(#kernel_1003)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 260.0 840.0\" to=\"360 260.0 840.0\" dur=\"0s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<text class=\"text\" x=\"288\" y=\"845.0\">rgba_rtp_thresh_2</text><linearGradient id=\"kernel_1004\" gradientTransform=\"rotate(90)\"> <stop offset=\"0%\" stop-color=\"white\"/> <stop offset=\"10%\" stop-color=\"#DC267F\"/> </linearGradient> <circle id=\"kernel_circle_1004\" class=\"kernel\" cx=\"260.0\" cy=\"790.0\" r=\"20\" fill=\"url(#kernel_1004)\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"5, 5\"> <animateTransform attributeName=\"transform\" type=\"rotate\" from=\"0 260.0 790.0\" to=\"360 260.0 790.0\" dur=\"0s\" begin=\"0s\" repeatCount=\"indefinite\"/> </circle> \n",
       "<text class=\"text\" x=\"288\" y=\"795.0\">rgba_rtp_thresh_3</text></g></svg>"
      ],
      "text/plain": [
       "<IPython.core.display.SVG object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "app_builder = ScaledUpThresholdApplication()\n",
    "app_builder(x_in, x_out)\n",
    "app_builder.display()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "60c89771",
   "metadata": {},
   "source": [
    "From the visualization we can see the effect of the split and concatenate operations. Initially, data is loaded into the memory tile's local data memory. The buffers start to fill up (shown in **green**). As they begin to empty, which represents the data streaming to each of the compute tiles, they change color to match the software kernel/compute tile each buffer is streamed to. \n",
    "\n",
    "The output data follows a similar pattern, but in reverse. Data from each of the compute tiles is written to the local data memory in the memory tile. As the buffers fill, the colors correspond to the software kernel that generated the data. Once one set of buffers is full, the data gets concatenated before it is streamed out of the memory tile, emptying the buffers. The colors of these buffers change to **purple** to represent the concatenation of the data as it is streamed out.\n",
    "\n",
    "Note that the visualization is conceptual and does not necessarily represent the exact size or number of physical buffers that are used in this example. \n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "7a75d7dd-6613-49f3-8553-bb4a6b7a6f6a",
   "metadata": {},
   "source": [
    "## Build the Memory Tile Example"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "48358139-9272-4bc8-91c3-4eee1fd64a32",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Using cached rgba_rtp_thresh kernel object file...\n",
      "Building the xclbin...\n",
      "Successfully Building Application... ScaledUpThresholdApplication.xclbin & ScaledUpThresholdApplication.seq delivered\n"
     ]
    }
   ],
   "source": [
    "app_builder.build(x_in, x_out)"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "c97d9259-45c5-424c-8b59-581a55fbd64d",
   "metadata": {},
   "source": [
    "## Run the Memory Tile Example\n",
    "\n",
    "The application will now be loaded to the NPU using the `ImageLooper720p` visualization graph template.\n",
    "\n",
    "The XCLBIN file contains the binary executable files for each tile, and the binary file to load the configuration to the NPU.\n",
    "\n",
    "There are four sets of threshold parameters, each corresponding to a compute tile. RGB sliders will be added for each of the four software kernels. "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "id": "15b379d7-4ce0-4507-9721-7ddca1e77e77",
   "metadata": {
    "tags": []
   },
   "outputs": [],
   "source": [
    "from npu.lib.graphs.image_looper_720p import ImageLooper720p\n",
    "\n",
    "app = ImageLooper720p(img='images/jpg/ryzenai_future_starts_now.jpg', \n",
    "                      xclbin='ScaledUpThresholdApplication.xclbin', \n",
    "                      rtps={\"r_thresh\" : { \"type\": \"slider\", \"min\": 0, \"max\" : 255},\n",
    "                            \"g_thresh\" : { \"type\": \"slider\", \"min\": 0, \"max\" : 255},\n",
    "                            \"b_thresh\" : { \"type\": \"slider\", \"min\": 0, \"max\" : 255}})"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "2bbdbfbb-6308-4503-b89c-f8964012390b",
   "metadata": {},
   "source": [
    "When you run the application, you will see the tiling effect referred to earlier. Each compute tile is responsible for processing one quarter of the image. By varying the threshold values, you can see the image is divided into four columns, each processed by a different compute tile. "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "id": "b702b1f5-fbfa-4d74-ae9a-4cf9e23bbb20",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "application/vnd.jupyter.widget-view+json": {
       "model_id": "6d2deb6727134680b463a39e84fff0d1",
       "version_major": 2,
       "version_minor": 0
      },
      "text/plain": [
       "Image(value=b'', format='jpeg')"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "application/vnd.jupyter.widget-view+json": {
       "model_id": "cf8eb1e59ab14fad8a92421c1d368a48",
       "version_major": 2,
       "version_minor": 0
      },
      "text/plain": [
       "Button(description='Stop', style=ButtonStyle())"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "application/vnd.jupyter.widget-view+json": {
       "model_id": "bd98d5f8e78e40a8a89f7705571f8146",
       "version_major": 2,
       "version_minor": 0
      },
      "text/plain": [
       "HBox(children=(IntSlider(value=51, description='r_thresh', max=255, style=SliderStyle(description_width='auto'…"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "application/vnd.jupyter.widget-view+json": {
       "model_id": "7df5fe22cbe64411a341b934c2f46227",
       "version_major": 2,
       "version_minor": 0
      },
      "text/plain": [
       "HBox(children=(IntSlider(value=149, description='r_thresh', max=255, style=SliderStyle(description_width='auto…"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "application/vnd.jupyter.widget-view+json": {
       "model_id": "27670eebffbf4a6badbd558ad308a313",
       "version_major": 2,
       "version_minor": 0
      },
      "text/plain": [
       "HBox(children=(IntSlider(value=128, description='r_thresh', max=255, style=SliderStyle(description_width='auto…"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "application/vnd.jupyter.widget-view+json": {
       "model_id": "dd7a7d6e514146569647f0b4a4ee92b6",
       "version_major": 2,
       "version_minor": 0
      },
      "text/plain": [
       "HBox(children=(IntSlider(value=19, description='r_thresh', max=255, style=SliderStyle(description_width='auto'…"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "app.start()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "caac8e24",
   "metadata": {},
   "source": [
    "\n",
    "---\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "975a6a52",
   "metadata": {},
   "source": [
    "## Exercise for the Reader:\n",
    "\n",
    "Build your own scaled application (using `MTSplit`, `MTConcat`) by combining two different kernels: `RgbaInverse()` and `RgbaRtpThres()`. The data on even indices of the `MTSplit` should be processed by `RgbaInverse`, whereas data on odd indices of the `MTSplit` should be processed by `RgbaRtpThres`."
   ]
  },
  {
   "cell_type": "markdown",
   "id": "6a213f54",
   "metadata": {},
   "source": [
    "### Define your scaled application\n",
    "\n",
    "Import the necessary components and define your `callgraph`."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "7a1be63f",
   "metadata": {},
   "outputs": [],
   "source": [
    "import numpy as np\n",
    "\n",
    "from npu.build.appbuilder import AppBuilder\n",
    "from npu.build.mtkernel import MTSplit, MTConcat\n",
    "from npu.lib import ... # Import the RgbaInverse and RgbaRtpThres\n",
    "\n",
    "\n",
    "class ScaledUpExerciseApplication(AppBuilder):\n",
    "\n",
    "    def __init__(self):\n",
    "        # create list of kernels with `RgbaRtpThres` in the even indices and RgbaInverse in the odd indices\n",
    "        self.kernel_list = ... #<YOUR CODE GOES HERE>\n",
    "        self.mtbsplit = MTSplit(4)\n",
    "        self.mtbconcat = MTConcat()\n",
    "        super().__init__()\n",
    "\n",
    "    def callgraph(self, x_in: np.ndarray, x_out: np.ndarray) -> None:\n",
    "        kernel_outputs = [None] * 4\n",
    "\n",
    "        rows = x_in.shape[0]\n",
    "        bytes_per_row = x_in.shape[1] * x_in.shape[2]\n",
    "        for row in range(rows):\n",
    "            inputs = self.mtbsplit(x_in[row])\n",
    "            for i in range(4):\n",
    "                if (i%2) == 0:\n",
    "                    # call the RgbaRtpThres kernel\n",
    "                    #<YOUR CODE GOES HERE>\n",
    "                else:\n",
    "                    # call the RgbaInverse kernel\n",
    "                    #<YOUR CODE GOES HERE>\n",
    "            mtbuffer_out = self.mtbconcat(kernel_outputs)\n",
    "            x_out[row] = mtbuffer_out"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "44503810",
   "metadata": {},
   "source": [
    "Define input and output buffers, visualize and build your application."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "c6af8b43",
   "metadata": {},
   "outputs": [],
   "source": [
    "x_in = np.zeros(shape=(720, 1280,4), dtype=np.uint8)\n",
    "x_out = np.zeros(shape=(720, 1280,4), dtype=np.uint8)\n",
    "\n",
    "app_builder = ScaledUpExerciseApplication()\n",
    "app_builder(x_in, x_out)\n",
    "\n",
    "app_builder.display()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "c6556e1d",
   "metadata": {},
   "outputs": [],
   "source": [
    "app_builder.build(x_in, x_out)"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "f1ea3d2b",
   "metadata": {},
   "source": [
    "### Run your custom application\n",
    "\n",
    "Once built, load your custom application in the NPU using the `ImageLooper720p` visualization helper."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "c9e41589",
   "metadata": {},
   "outputs": [],
   "source": [
    "from npu.lib.graphs.image_looper_720p import ImageLooper720p\n",
    "\n",
    "app = ImageLooper720p(img='images/jpg/ryzenai_future_starts_now.jpg', \n",
    "                      xclbin='ScaledUpExerciseApplication.xclbin', \n",
    "                      rtps={\"r_thresh\" : { \"type\": \"slider\", \"min\": 0, \"max\" : 255},\n",
    "                            \"g_thresh\" : { \"type\": \"slider\", \"min\": 0, \"max\" : 255},\n",
    "                            \"b_thresh\" : { \"type\": \"slider\", \"min\": 0, \"max\" : 255}})"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "8f3f5ff4",
   "metadata": {},
   "source": [
    "When you run the application, you should see that column 0 and 2 are the output of the `RgbaRtpThres` and column 1 and 3 are the output of `RgbaInverse`. By varying the threshold values, this effect should be noticeable. "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "f9fd735f",
   "metadata": {},
   "outputs": [],
   "source": [
    "app.start()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "589afe30",
   "metadata": {},
   "source": [
    "\n",
    "---\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "b5f0569d",
   "metadata": {},
   "source": [
    "## Next Steps\n",
    "\n",
    "In the next notebook you will learn how to build a pipeline dataflow application."
   ]
  },
  {
   "cell_type": "markdown",
   "id": "be636627",
   "metadata": {},
   "source": [
    "\n",
    "---\n",
    "\n",
    "<center>\n",
    "Copyright&copy; 2023 AMD, Inc\n",
    "</center>\n",
    "<center>\n",
    "SPDX-License-Identifier: MIT\n",
    "</center>"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.2"
  },
  "widgets": {
   "application/vnd.jupyter.widget-state+json": {
    "state": {},
    "version_major": 2,
    "version_minor": 0
   }
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
